%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Julio M. Otuyama - 10/1999
%
% convmkdft.m
%
% Matrix Convolution
% 
%   Recebe:
%		data - dados para convolucao
%		  dimensoes esperadas (nesta ordem):
%		    size_conv
%         size_same
%			 size_contract
%         size_expand_data
%
%		kernel - kernel da convolucao
%		  dimensoes esperadas (nesta ordem):
%		    size_conv
%         size_same
%			 size_contract
%         size_expand_kernel
%
%		dim - tamanho das dimensoes esperadas
%		  conv
%		  same
%		  contract
%       expand_data
%       expand_kernel
%
%   Retorna:
%		result - resultado da convolucao
%		  dimensoes retornadas (nesta ordem):
%		    size_conv
%		    size_same
%			 size_expand_data
%			 size_expand_kernel
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [result]=convmkdft(data,kerneldft,dim)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if((dim.conv<0)|(dim.same<0)|(dim.expand_data<0)|(dim.expand_kernel<0)|(dim.contract<0))
   error('convc: dimesao negativa invalida');
end;

%dim
%disp('data size');
%ndsize(data)
%disp('kernel size');
%ndsize(kernel)

size_data=ndSize(data);
size_kernel=ndSize(kerneldft);
size_dft={size_kernel{1:dim.conv}};
size_data_conv={size_data{1:dim.conv}};

% data -> datadft
data.data=fft0n(data.data,[size_dft{:}]);

% dim
dimprod.same=dim.conv+dim.same;
dimprod.contract=dim.contract;
dimprod.expand_A=dim.expand_data;
dimprod.expand_B=dim.expand_kernel;

result=ndprod(data,kerneldft,dimprod);

% resultdft -> result
tmp=real(ifft0n(result.data,[size_dft{:}]));

offset=floor(([size_dft{:}]+1-[size_data_conv{:}])/2);

ind_tmp_conv=ndRange(num2cell(offset+1),num2cell(offset+[size_data_conv{:}]));
ind_tmp_etc=ndAll(ndSize(result));
ind_tmp={ind_tmp_conv{:},ind_tmp_etc{dim.conv+1:end}};
result.data=tmp(ind_tmp{:});
